package cc.fascinated.bat.command.impl.global.beatsaber.scoresaber;

import cc.fascinated.bat.command.BatCommand;
import cc.fascinated.bat.common.DateUtils;
import cc.fascinated.bat.common.NumberUtils;
import cc.fascinated.bat.model.BatGuild;
import cc.fascinated.bat.model.beatsaber.scoresaber.ScoreSaberAccountToken;
import cc.fascinated.bat.model.user.BatUser;
import cc.fascinated.bat.model.user.profiles.ScoreSaberProfile;
import cc.fascinated.bat.service.ScoreSaberService;
import lombok.NonNull;
import net.dv8tion.jda.api.EmbedBuilder;
import net.dv8tion.jda.api.entities.Member;
import net.dv8tion.jda.api.entities.MessageEmbed;
import net.dv8tion.jda.api.entities.channel.concrete.TextChannel;
import net.dv8tion.jda.api.interactions.commands.OptionMapping;
import net.dv8tion.jda.api.interactions.commands.OptionType;
import net.dv8tion.jda.api.interactions.commands.SlashCommandInteraction;
import net.dv8tion.jda.api.interactions.commands.build.OptionData;
import net.dv8tion.jda.internal.interactions.CommandDataImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.time.LocalDateTime;

/**
 * @author Fascinated (fascinated7)
 */
@Component
public class ScoreSaberCommand extends BatCommand {
    private final ScoreSaberService scoreSaberService;

    @Autowired
    public ScoreSaberCommand(@NonNull ScoreSaberService scoreSaberService) {
        super("scoresaber");
        super.setCategory(Category.BEAT_SABER);
        this.scoreSaberService = scoreSaberService;

        super.setDescription("View a user's ScoreSaber profile");
        super.setCommandData(new CommandDataImpl(this.getName(), this.getDescription())
                .addOptions(new OptionData(OptionType.STRING, "link", "Link your ScoreSaber profile", false))
                .addOptions(new OptionData(OptionType.USER, "user", "The user to view the ScoreSaber profile of", false))
        );
    }

    @Override
    public void execute(@NonNull BatGuild guild, @NonNull BatUser user, @NonNull TextChannel channel, @NonNull Member member, @NonNull SlashCommandInteraction interaction, OptionMapping option) {
        ScoreSaberProfile profile = user.getProfile(ScoreSaberProfile.class);
        if (profile.getId() == null) {
            interaction.reply("You must link your ScoreSaber profile first").queue();
            return;
        }

        // todo: handle rate limits
        ScoreSaberAccountToken account = scoreSaberService.getAccount(profile.getId());
        if (account == null) {
            interaction.reply("Invalid ScoreSaber profile, please re-link your account.").queue();
            return;
        }

        interaction.replyEmbeds(buildProfileEmbed(account)).queue();
    }

    /**
     * Builds the profile embed for the ScoreSaber profile
     *
     * @param account The account to build the embed for
     * @return The built embed
     */
    public static MessageEmbed buildProfileEmbed(ScoreSaberAccountToken account) {
        return new EmbedBuilder()
                .setAuthor(account.getName() + "'s Profile", "https://scoresaber.com/u/%s".formatted(account.getId()),
                        "https://cdn.scoresaber.com/avatars/%s.jpg".formatted(account.getId()))
                .addField("Name", account.getName(), true)
                .addField("Country", account.getCountry(), true)
                .addField("Rank", "#" + account.getRank(), true)
                .addField("Country Rank", "#" + account.getCountryRank(), true)
                .addField("PP", NumberUtils.formatNumberCommas(account.getPp()), true)
                .addField("Joined", "<t:%s>".formatted(DateUtils.getDateFromString(account.getFirstSeen()).toInstant().toEpochMilli()/1000), true)
                .setTimestamp(LocalDateTime.now())
                .build();
    }
}
